<?php
namespace WSCOMUN\CATASTRO;
use SoapVar;
use DateTime;
use ArrayObject;

class objSolicitudTransmision
{
	/**
	 * Contantes para los NS
	 */
	private static $NAMESPACE_SCSP_ESPECIFICO = 'http://intermediacion.redsara.es/scsp/esquemas/datosespecificos';
	private static $NAMESPACE_CAT_PET = 'http://intermediacion.redsara.es/scsp/esquemas/V3/peticion';
	private static $NAMESPACE_CAT_SR = 'http://intermediacion.redsara.es/scsp/esquemas/V3/solicitudRespuesta';

	private $nifEmisor = null;
	private $nombreEmisor = null;
	public function setEmisor($nif, $nombre) {
		$this->nifEmisor = $nif;
		$this->nombreEmisor = $nombre;
	}

	private $codProcedimiento = null;
	private $nombreProcedimiento = null;
	public function setProcedimiento($codigo, $nombre) {
		$this->codProcedimiento = $codigo;
		$this->nombreProcedimiento = $nombre;
	}

	private $nombreCompletoFuncionario = null;
	private $nifFuncionario = null;
	public function setFuncionario($nombreCompleto, $nif) {
		$this->nombreCompletoFuncionario = $nombreCompleto;
		$this->nifFuncionario = $nif;
	}

	private $tipoDocumentacion = null;
	private $documentacion = null;
	private $nombreCompleto = null;
	private $nombre = null;
	private $apellido1 = null;
	private $apellido2 = null;
	public function setTitular($tipoDocumentacion, $documentacion, $nombreCompleto = null, $nombre = null, $apellido1 = null, $apellido2 = null) {
		$this->tipoDocumentacion = $tipoDocumentacion;
		$this->documentacion = $documentacion;
		$this->nombreCompleto = $nombreCompleto;
		$this->nombre = $nombre;
		$this->apellido1 = $apellido1;
		$this->apellido2 = $apellido2;
	}

	private $identificadorSolicitante = null;
	private $nombreSolicitante = null;
	private $unidadTramitadora = null;
	private $finalidad = null;
	private $consentimiento = null;
	private $idExpediente = null;
	public function setSolicitante($identificadorSolicitante, $nombreSolicitante, $consentimiento, $finalidad, $unidadTramitadora = null, $idExpediente = null) {
		$this->identificadorSolicitante = $identificadorSolicitante;
		$this->nombreSolicitante = $nombreSolicitante;
		$this->consentimiento = $consentimiento;
		$this->finalidad = $finalidad;
		$this->unidadTramitadora = $unidadTramitadora;
		$this->idExpediente = $idExpediente;
	}

	private $oEspecifico = null;
	public function setEspecifico($oEspecifico) {
		$this->oEspecifico = $oEspecifico;
	}

	public function duplicarCabecera() {
		$oClon = new objSolicitudTransmision();
		$oClon->setEmisor($this->nifEmisor, $this->nombreEmisor);
		$oClon->setProcedimiento($this->codProcedimiento, $this->nombreProcedimiento);
		$oClon->setFuncionario($this->nombreCompletoFuncionario, $this->nifFuncionario);
		$oClon->setTitular($this->tipoDocumentacion, $this->documentacion, $this->nombreCompleto, $this->nombre, $this->apellido1, $this->apellido2);
		$oClon->setSolicitante($this->identificadorSolicitante, $this->nombreSolicitante, $this->consentimiento, $this->finalidad, $this->unidadTramitadora, $this->idExpediente);

		return $oClon;
	}

	public function toSoap($IdPeticion, $CodigoCertificado, $numParcial = null)
	{
		// TODO: Sincrona
		$IdSolicitud = $IdPeticion;
		if (!is_null($numParcial)) {
			$IdSolicitud = $IdPeticion . '_' . $numParcial;
		}

		// $CodigoCertificado debera ser auto-generado
		// IdentificadorSolicitante = (Sugerido) CODxxxxxx
		// Consentimiento = Si | Ley
		// TipoDocumentacion = NIF | CIF | DNI | Pasaporte | NIE

		// Emisor
		$EmisorWSRequest = new ArrayObject();
		$EmisorWSRequest->append(new SoapVar($this->nifEmisor, XSD_STRING, null, null,'NifEmisor', self::$NAMESPACE_CAT_PET));
		$EmisorWSRequest->append(new SoapVar($this->nombreEmisor, XSD_STRING, null, null,'NombreEmisor', self::$NAMESPACE_CAT_PET));

		// Procedimiento
		$ProcedimientoWSRequest = new ArrayObject();
		$ProcedimientoWSRequest->append(new SoapVar($this->codProcedimiento, XSD_STRING, null, null,'CodProcedimiento', self::$NAMESPACE_CAT_PET));
		$ProcedimientoWSRequest->append(new SoapVar($this->nombreProcedimiento, XSD_STRING, null, null,'NombreProcedimiento', self::$NAMESPACE_CAT_PET));

		// Funcionario
		$FuncionarioWSRequest = new ArrayObject();
		$FuncionarioWSRequest->append(new SoapVar($this->nombreCompletoFuncionario, XSD_STRING, null, null,'NombreCompletoFuncionario', self::$NAMESPACE_CAT_PET));
		$FuncionarioWSRequest->append(new SoapVar($this->nifFuncionario, XSD_STRING, null, null,'NifFuncionario', self::$NAMESPACE_CAT_PET));

		// Solicitante
		$SolicitanteWSRequest = new ArrayObject();
		$SolicitanteWSRequest->append(new SoapVar($this->identificadorSolicitante, XSD_STRING, null, null,'IdentificadorSolicitante', self::$NAMESPACE_CAT_PET));
		$SolicitanteWSRequest->append(new SoapVar($this->nombreSolicitante, XSD_STRING, null, null,'NombreSolicitante', self::$NAMESPACE_CAT_PET));
		if (!is_null($this->unidadTramitadora)) {
			$SolicitanteWSRequest->append(new SoapVar($this->unidadTramitadora, XSD_STRING, null, null,'UnidadTramitadora', self::$NAMESPACE_CAT_PET));
		}
		$SolicitanteWSRequest->append(new SoapVar($ProcedimientoWSRequest, SOAP_ENC_OBJECT,null,null,'Procedimiento', self::$NAMESPACE_CAT_PET));
		$SolicitanteWSRequest->append(new SoapVar($this->finalidad, XSD_STRING, null, null,'Finalidad', self::$NAMESPACE_CAT_PET));
		$SolicitanteWSRequest->append(new SoapVar($this->consentimiento, XSD_STRING, null, null,'Consentimiento', self::$NAMESPACE_CAT_PET));
		$SolicitanteWSRequest->append(new SoapVar($FuncionarioWSRequest, SOAP_ENC_OBJECT,null,null,'Funcionario', self::$NAMESPACE_CAT_PET));
		if (!is_null($this->idExpediente)) {
			$SolicitanteWSRequest->append(new SoapVar($this->idExpediente, XSD_STRING, null, null,'IdExpediente', self::$NAMESPACE_CAT_PET));
		}

		// Titular
		$TitularWSRequest = new ArrayObject();
		$TitularWSRequest->append(new SoapVar($this->tipoDocumentacion, XSD_STRING, null, null,'TipoDocumentacion', self::$NAMESPACE_CAT_PET));
		$TitularWSRequest->append(new SoapVar($this->documentacion, XSD_STRING, null, null,'Documentacion', self::$NAMESPACE_CAT_PET));
		if (!is_null($this->nombreCompleto)) {
			$TitularWSRequest->append(new SoapVar($this->nombreCompleto, XSD_STRING, null, null,'NombreCompleto', self::$NAMESPACE_CAT_PET));
		}
		if (!is_null($this->nombre)) {
			$TitularWSRequest->append(new SoapVar($this->nombre, XSD_STRING, null, null,'Nombre', self::$NAMESPACE_CAT_PET));
		}
		if (!is_null($this->apellido1)) {
			$TitularWSRequest->append(new SoapVar($this->apellido1, XSD_STRING, null, null,'Apellido1', self::$NAMESPACE_CAT_PET));
		}
		if (!is_null($this->apellido2)) {
			$TitularWSRequest->append(new SoapVar($this->apellido2, XSD_STRING, null, null,'Apellido2', self::$NAMESPACE_CAT_PET));
		}

		// Transmision
		$TransmisionWSRequest = new ArrayObject();
		$TransmisionWSRequest->append(new SoapVar($CodigoCertificado, XSD_STRING, null, null,'CodigoCertificado', self::$NAMESPACE_CAT_PET));
		$TransmisionWSRequest->append(new SoapVar($IdSolicitud, XSD_STRING, null, null,'IdSolicitud', self::$NAMESPACE_CAT_PET));

		// DatosGenericos
		$DatosGenericosWSRequest = new ArrayObject();
		$DatosGenericosWSRequest->append(new SoapVar($EmisorWSRequest, SOAP_ENC_OBJECT,null,null,'Emisor', self::$NAMESPACE_CAT_PET));
		$DatosGenericosWSRequest->append(new SoapVar($SolicitanteWSRequest, SOAP_ENC_OBJECT,null,null,'Solicitante', self::$NAMESPACE_CAT_PET));
		$DatosGenericosWSRequest->append(new SoapVar($TitularWSRequest, SOAP_ENC_OBJECT,null,null,'Titular', self::$NAMESPACE_CAT_PET));
		$DatosGenericosWSRequest->append(new SoapVar($TransmisionWSRequest, SOAP_ENC_OBJECT,null,null,'Transmision', self::$NAMESPACE_CAT_PET));

		// SolicitudTransmision
		$SolicitudTransmisionWSRequest = new ArrayObject();
		$SolicitudTransmisionWSRequest->append(new SoapVar($DatosGenericosWSRequest, SOAP_ENC_OBJECT,null,null,'DatosGenericos', self::$NAMESPACE_CAT_PET));


		if (!is_null($this->oEspecifico)) {
			$datosEspecificosSoapVar = $this->oEspecifico->toSoap();
			$SolicitudTransmisionWSRequest->append($datosEspecificosSoapVar);
		}


		// Devuelve la solicitud ya generada
		return new SoapVar($SolicitudTransmisionWSRequest, SOAP_ENC_OBJECT,null,null,'SolicitudTransmision', self::$NAMESPACE_CAT_PET);
	}

}


class objCatastro
{
	/**
	 * Contantes para los NS
	 */
	private static $NAMESPACE_SCSP_ESPECIFICO = 'http://intermediacion.redsara.es/scsp/esquemas/datosespecificos';
	private static $NAMESPACE_CAT_PET = 'http://intermediacion.redsara.es/scsp/esquemas/V3/peticion';
	private static $NAMESPACE_CAT_SR = 'http://intermediacion.redsara.es/scsp/esquemas/V3/solicitudRespuesta';


	private function _doPeticion($idPeticion, $numElementos, $timestamp, $codigoCertificado, $ns = null) {
		if (is_null($ns)) {
			$ns = self::$NAMESPACE_CAT_PET;
		}

		// Atributos
		$AtrWSLvl = new ArrayObject();
		$AtrWSLvl->append(new SoapVar($idPeticion, XSD_STRING, null, null,'IdPeticion', $ns));
		$AtrWSLvl->append(new SoapVar($numElementos, XSD_INTEGER , null, null,'NumElementos', $ns));
		$AtrWSLvl->append(new SoapVar($timestamp->format(DateTime::ATOM), XSD_STRING, null, null,'TimeStamp', $ns));
		$AtrWSLvl->append(new SoapVar($codigoCertificado, XSD_STRING, null, null,'CodigoCertificado', $ns));

		return new SoapVar($AtrWSLvl, SOAP_ENC_OBJECT, null, null, 'Atributos', $ns);
	}


	public function doGeneral($IdPeticion, $TimeStamp, $CodigoCertificado, $oSolicitud)
	{
		// $CodigoCertificado debera ser auto-generado
		// IdentificadorSolicitante = (Sugerido) CODxxxxxx
		// Consentimiento = Si | Ley
		// TipoDocumentacion = NIF | CIF | DNI | Pasaporte | NIE

		// Atributos
		$AtrSoapVar = $this->_doPeticion($IdPeticion, 1, $TimeStamp, $CodigoCertificado);

		// SolicitudTransmision
		$SolicitudTransmisionSoapVar = $oSolicitud->toSoap($IdPeticion, $CodigoCertificado, null);

		// Solicitudes
		$SolicitudesWSRequest = new ArrayObject();
		$SolicitudesWSRequest->append($SolicitudTransmisionSoapVar);

		// Body
		$bodyWSRequest = new ArrayObject();
		$bodyWSRequest->append($AtrSoapVar);
		$bodyWSRequest->append(new SoapVar($SolicitudesWSRequest, SOAP_ENC_OBJECT, null, null, 'Solicitudes', self::$NAMESPACE_CAT_PET));

		return $bodyWSRequest;
	}//catastro



	public function catastroBienesPeticion($IdPeticion, $TimeStamp, $CodigoCertificado, $vSolicitudes)
	{
		// $CodigoCertificado debera ser auto-generado
		// IdentificadorSolicitante = (Sugerido) CODxxxxxx
		// Consentimiento = Si | Ley
		// TipoDocumentacion = NIF | CIF | DNI | Pasaporte | NIE

		// Cuenta el numero de solicitudes
		$NumElementos = count($vSolicitudes);

		// Solicitudes
		$SolicitudesWSRequest = new ArrayObject();

		for ($iPet=0; $iPet < $NumElementos; $iPet++) {
			$oSolicitud = $vSolicitudes[$iPet];

			// SolicitudTransmision
			$SolicitudTransmisionSoapVar = $oSolicitud->toSoap($IdPeticion, $CodigoCertificado, $iPet);

			// Aade la solicitud de transmision al listado de solicitudes
			$SolicitudesWSRequest->append($SolicitudTransmisionSoapVar);
		}

		// Atributos
		$AtrSoapVar = $this->_doPeticion($IdPeticion, $NumElementos, $TimeStamp, $CodigoCertificado);

		// Body
		$bodyWSRequest = new ArrayObject();
		$bodyWSRequest->append($AtrSoapVar);
		$bodyWSRequest->append(new SoapVar($SolicitudesWSRequest, SOAP_ENC_OBJECT,null,null,'Solicitudes', self::$NAMESPACE_CAT_PET));

		return $bodyWSRequest;
	}//catastroBienesPeticion

	public function catastroBienesRespuesta($IdPeticion, $TimeStamp, $CodigoCertificado, $NumElementos)
	{
		// Atributos
		$AtrSoapVar = $this->_doPeticion($IdPeticion, $NumElementos, $TimeStamp, $CodigoCertificado, self::$NAMESPACE_CAT_SR);

		// Body
		$bodyWSRequest = new ArrayObject();
		$bodyWSRequest->append($AtrSoapVar);

		return $bodyWSRequest;
	}//catastroBienesRespuesta

}



class BienesEspecifico {

	private static $NAMESPACE_CAT_ESP = 'http://intermediacion.redsara.es/scsp/esquemas/datosespecificos';

	private $esRefCatastral = true;

	private $pc1 = null;
	private $pc2 = null;
	private $car = null;
	private $cc1 = null;
	private $cc2 = null;

	private $cp = null;
	private $cm = null;
	private $cpo = null;
	private $cpa = null;

	private function __construct($esRefCatastral) {
		$this->esRefCatastral = $esRefCatastral;
	}

	static public function referenciaCatastral($pc1, $pc2, $car = null, $cc1 = null, $cc2 = null) {
		$oEsp = new BienesEspecifico(true);

		$oEsp->pc1 = $pc1;
		$oEsp->pc2 = $pc2;
		$oEsp->car = $car;
		$oEsp->cc1 = $cc1;
		$oEsp->cc2 = $cc2;

		return $oEsp;
	}

	static public function referenciaRustica($cp, $cm, $cpo, $cpa = null) {
		$oEsp = new BienesEspecifico(false);

		$oEsp->cp = $cp;
		$oEsp->cm = $cm;
		$oEsp->cpo = $cpo;
		$oEsp->cpa = $cpa;

		return $oEsp;
	}

	public function toSoap() {
		$datosEntradaLvl = null;
		if ($this->esRefCatastral) {
			$datosEntradaLvl = $this->_toSoap_RefCatastral();
		} else {
			$datosEntradaLvl = $this->_toSoap_RefRustica();
		}

		$datosEspLvl = new ArrayObject();
		$datosEspLvl->append(new SoapVar($datosEntradaLvl, SOAP_ENC_OBJECT, null, null, 'DatosEntrada', self::$NAMESPACE_CAT_ESP));

		$mainSoapVar = new SoapVar($datosEspLvl, SOAP_ENC_OBJECT, null, null, 'DatosEspecificos', self::$NAMESPACE_CAT_ESP);
		return $mainSoapVar;
		/*
		 $mainLvl = new ArrayObject();
		 $mainLvl->append();
		 return $mainLvl;
		 */
	}

	private function _toSoap_RefCatastral() {
		$refCatLvl = new ArrayObject();
		$refCatLvl->append(new SoapVar($this->pc1, XSD_STRING, null, null, 'pc1', self::$NAMESPACE_CAT_ESP));
		$refCatLvl->append(new SoapVar($this->pc2, XSD_STRING, null, null, 'pc2', self::$NAMESPACE_CAT_ESP));
		if (!is_null($this->car)) {
			$refCatLvl->append(new SoapVar($this->car, XSD_STRING, null, null, 'car', self::$NAMESPACE_CAT_ESP));
		}
		if (!is_null($this->cc1)) {
			$refCatLvl->append(new SoapVar($this->cc1, XSD_STRING, null, null, 'cc1', self::$NAMESPACE_CAT_ESP));
		}
		if (!is_null($this->cc2)) {
			$refCatLvl->append(new SoapVar($this->cc2, XSD_STRING, null, null, 'cc2', self::$NAMESPACE_CAT_ESP));
		}

		$datosEntradaLvl = new ArrayObject();
		$datosEntradaLvl->append(new SoapVar($refCatLvl, SOAP_ENC_OBJECT,null,null, 'referenciaCatastral', self::$NAMESPACE_CAT_ESP));

		return $datosEntradaLvl;
	}

	private function _toSoap_RefRustica() {
		$loINELvl = new ArrayObject();
		$loINELvl->append(new SoapVar($this->cp, XSD_STRING, null, null, 'cp', self::$NAMESPACE_CAT_ESP));
		$loINELvl->append(new SoapVar($this->cm, XSD_STRING, null, null, 'cm', self::$NAMESPACE_CAT_ESP));

		$cppLvl = new ArrayObject();
		$cppLvl->append(new SoapVar($this->cpo, XSD_STRING, null, null, 'cpo', self::$NAMESPACE_CAT_ESP));
		if (!is_null($this->cpa)) {
			$cppLvl->append(new SoapVar($this->cpa, XSD_STRING, null, null, 'cpa', self::$NAMESPACE_CAT_ESP));
		}

		$refRustLvl = new ArrayObject();
		$refRustLvl->append(new SoapVar($loINELvl, SOAP_ENC_OBJECT, null, null, 'localizacionINE', self::$NAMESPACE_CAT_ESP));
		$refRustLvl->append(new SoapVar($cppLvl, SOAP_ENC_OBJECT, null, null, 'cpp', self::$NAMESPACE_CAT_ESP));

		$datosEntradaLvl = new ArrayObject();
		$datosEntradaLvl->append(new SoapVar($refRustLvl, SOAP_ENC_OBJECT,null,null, 'referenciaRustica', self::$NAMESPACE_CAT_ESP));

		return $datosEntradaLvl;
	}

}

?>