<?php
/**
 * Librera para que aplicaciones PHP/GVHidra puedan acceder al Web Service de Firma (autorizacionArangi)
 *
 * @package		WSClientFirma
 * @author 		David Pascual <pascual_dav@externosgva.es>
 * @version		Release: @package_version@
 * @copyright 	Copyright (C) 2015 Generalitat Valenciana.
 * @license 	http://opensource.org/licenses/gpl-2.0.php GNU Public License v2.0
 * @link 		https://gvhidra.gva.es/redmine/projects/wscomun
 */

use WSCOMUN\WSComunFClient;

class WSClientFirma extends WSComunFClient
{

	/**
	 * Permite firmar con el cert. de servidor SAFE un documento (con formato concreto).
	 *
	 * Los WS invocados pertenecen al servicio deAutorizacin) (autorizacionArangi v1.00).
	 * @subpackage 	Autorizacion
	 * @param  	string	$idSession	Identificador SAFE de APP/Sesion (ej. TRA)
	 * @param  	integer	$idCertificado	Identificador del certificado ubicado en SAFE (ej. 5)
	 * @param  	string	$docContentB64	Array de bytes a firmar (contenido del documento) en base 64
	 * @param  	string	$tipoDoc	Tipo del documento a firma (PDF por defecto)
	 * @return	string	Devuelve el contenido BINARIO firmado
	 * @access 	public
	 */
	public function firmarCertificadoServer ($idSession, $idCertificado, $docContentB64, $tipoDoc='TF06')
	{
		try
		{
			$this->__clienteOn(self::KEYCLIENTE_FIRMA);
			$clienteWS = $this->v_clienteWS[self::KEYCLIENTE_FIRMA];


			$firmarCertificadoWSRequest = new ArrayObject();
			$firmarCertificadoWSRequest->append(new SoapVar($idSession, XSD_STRING, null, null,'idSession', self::$NAMESPACE_GVA));
			$firmarCertificadoWSRequest->append(new SoapVar($idCertificado, XSD_STRING, null, null,'idCertificado', self::$NAMESPACE_GVA));
			$firmarCertificadoWSRequest->append(new SoapVar($docContentB64, XSD_STRING, null, null,'documento', self::$NAMESPACE_GVA));
			$firmarCertificadoWSRequest->append(new SoapVar('TF06', XSD_STRING, null, null, 'firmaFormato', self::$NAMESPACE_GVA));
			$firmarCertificadoWSRequest->append(new SoapVar('pdf', XSD_STRING, null, null, 'formatoSubtipo', self::$NAMESPACE_GVA));//pades-ltv o pdf
			$miSoapRQ = new SoapVar($firmarCertificadoWSRequest, SOAP_ENC_OBJECT,null,null,'firmarCertificadoWSRequest', self::$NAMESPACE_GVA);
			$docFirmado = $clienteWS->firmarConCertificado($miSoapRQ);

			if (is_object($docFirmado))
			{
				return ($docFirmado->response);
			}
			else
			{
				return($docFirmado);
			}
		}
		catch (Exception $e)
		{
			if ($this->getDebugMode())
			{
				$this->addDebugInfo(__CLASS__.':'.__METHOD__.'['.__FILE__.'-'.__LINE__.']'."\n");
				if (is_object($clienteWS))
				{
					$this->addDebugInfo("RqH:\n ".$clienteWS->__getLastRequestHeaders());
					$this->addDebugInfo("Rq:\n ".$clienteWS->__getLastRequest());
					$this->addDebugInfo("RsH:\n ".$clienteWS->__getLastResponse());
					$this->addDebugInfo("Rs:\n ".$clienteWS->__getLastResponseHeaders());
				}
				throw $e;
			}
			else
			{
				throw $e;
			}
		}
	}//firmarCertificadoServer

}



?>